@@ -16,7 +16,7 @@ Features:
 - Errors reporting.
 
 Limitations:
-- Byte enable not supported for writes.
+- Write byte enable granularity of DRAM's data-width.
 """
 
 from migen import *
@@ -31,43 +31,69 @@ from litedram.common import wdata_description, rdata_description
 # LiteDRAMNativePortECCW ---------------------------------------------------------------------------
 
 class LiteDRAMNativePortECCW(Module):
-    def __init__(self, data_width_from, data_width_to):
-        self.sink   = sink   = Endpoint(wdata_description(data_width_from))
-        self.source = source = Endpoint(wdata_description(data_width_to))
+    def __init__(self, data_width_from, data_width_to, burst_cycles=8):
+        self.sink     = sink   = Endpoint(wdata_description(data_width_from))
+        self.source   = source = Endpoint(wdata_description(data_width_to))
+        self.we_error = Signal()
 
         # # #
 
-        for i in range(8):
-            encoder = ECCEncoder(data_width_from//8)
-            self.submodules += encoder
+        # Control (ECC encoding is combinatorial).
+        self.comb += sink.connect(source, omit={"data", "we"}),
+
+        # Data Path.
+        for i in range(burst_cycles):
+            ecc_width_from = data_width_from // burst_cycles
+            ecc_width_to   = data_width_to   // burst_cycles
+
+            # Encoder.
+            self.submodules.encoder = encoder = ECCEncoder(ecc_width_from)
             self.comb += [
-                sink.connect(source, omit={"data", "we"}),
-                encoder.i.eq(sink.data[i*data_width_from//8:(i+1)*data_width_from//8]),
-                source.data[i*data_width_to//8:(i+1)*data_width_to//8].eq(encoder.o)
+                # Input.
+                encoder.i.eq(sink.data[i*ecc_width_from:(i+1)*ecc_width_from]),
+                # Output.
+                # We always have to store the full ECC-Word so byte enable is not supported within
+                # an ECC word. If any of the byte enable is set, the full ECC-Word is written.
+                If(sink.we[i*ecc_width_from//8:(i+1)*ecc_width_from//8] != 0,
+                    source.we[i*ecc_width_to//8:(i+1)*ecc_width_to//8].eq(2**ecc_width_to//8-1)
+                ),
+                source.data[i*ecc_width_to:(i+1)*ecc_width_to].eq(encoder.o),
+                # Byte enable granularity  error detection.
+                If(sink.valid & (sink.we[i*ecc_width_from//8:(i+1)*ecc_width_from//8] != (2**ecc_width_from//8-1)),
+                    self.we_error.eq(1)
+                )
             ]
-        self.comb += source.we.eq(2**len(source.we)-1)
 
 # LiteDRAMNativePortECCR ---------------------------------------------------------------------------
 
 class LiteDRAMNativePortECCR(Module):
-    def __init__(self, data_width_from, data_width_to):
+    def __init__(self, data_width_from, data_width_to, burst_cycles=8):
         self.sink   = sink   = Endpoint(rdata_description(data_width_to))
         self.source = source = Endpoint(rdata_description(data_width_from))
         self.enable = Signal()
-        self.sec    = Signal(8)
-        self.ded    = Signal(8)
+        self.sec    = Signal(burst_cycles)
+        self.ded    = Signal(burst_cycles)
 
         # # #
 
+        # Control Path (ECC decoding is combinatorial).
         self.comb +=  sink.connect(source, omit={"data"})
 
-        for i in range(8):
-            decoder = ECCDecoder(data_width_from//8)
-            self.submodules += decoder
+        # Data Path.
+        for i in range(burst_cycles):
+            ecc_width_to   = data_width_to   // burst_cycles
+            ecc_width_from = data_width_from // burst_cycles
+
+            # Decoder.
+            self.submodules.decoder = decoder = ECCDecoder(ecc_width_from)
             self.comb += [
+                # Enable.
                 decoder.enable.eq(self.enable),
-                decoder.i.eq(sink.data[i*data_width_to//8:(i+1)*data_width_to//8]),
-                source.data[i*data_width_from//8:(i+1)*data_width_from//8].eq(decoder.o),
+                # Input.
+                decoder.i.eq(sink.data[i*ecc_width_to:(i+1)*ecc_width_to]),
+                # Output.
+                source.data[i*ecc_width_from:(i+1)*ecc_width_from].eq(decoder.o),
+                # Bitflip/Error reporting.
                 If(source.valid,
                     self.sec[i].eq(decoder.sec),
                     self.ded[i].eq(decoder.ded)
@@ -77,9 +103,9 @@ class LiteDRAMNativePortECCR(Module):
 # LiteDRAMNativePortECC ----------------------------------------------------------------------------
 
 class LiteDRAMNativePortECC(Module, AutoCSR):
-    def __init__(self, port_from, port_to, with_error_injection=False):
-        _ , n = compute_m_n(port_from.data_width//8)
-        assert port_to.data_width >= (n + 1)*8
+    def __init__(self, port_from, port_to, burst_cycles=8, with_error_injection=False, with_we_error_detection=False):
+        _ , n = compute_m_n(port_from.data_width//burst_cycles)
+        assert port_to.data_width >= (n + 1)*burst_cycles
 
         self.enable     = CSRStorage(reset=1)
         self.clear      = CSR()
@@ -89,14 +115,16 @@ class LiteDRAMNativePortECC(Module, AutoCSR):
         self.ded_detected = ded_detected = Signal()
         if with_error_injection:
             self.flip = CSRStorage(8)
+        if with_we_error_detection:
+            self.we_errors = CSRStatus(32)
 
         # # #
 
         # Cmd --------------------------------------------------------------------------------------
         self.comb += port_from.cmd.connect(port_to.cmd)
 
-        # Wdata (ecc encoding) ---------------------------------------------------------------------
-        ecc_wdata = LiteDRAMNativePortECCW(port_from.data_width, port_to.data_width)
+        # Wdata (ECC) encoding) --------------------------------------------------------------------
+        ecc_wdata = LiteDRAMNativePortECCW(port_from.data_width, port_to.data_width, burst_cycles)
         ecc_wdata = BufferizeEndpoints({"source": DIR_SOURCE})(ecc_wdata)
         self.submodules += ecc_wdata
         self.comb += [
@@ -106,10 +134,10 @@ class LiteDRAMNativePortECC(Module, AutoCSR):
         if with_error_injection:
             self.comb += port_to.wdata.data[:8].eq(self.flip.storage ^ ecc_wdata.source.data[:8])
 
-        # Rdata (ecc decoding) ---------------------------------------------------------------------
+        # Rdata (ECC decoding) ---------------------------------------------------------------------
         sec = Signal()
         ded = Signal()
-        ecc_rdata = LiteDRAMNativePortECCR(port_from.data_width, port_to.data_width)
+        ecc_rdata = LiteDRAMNativePortECCR(port_from.data_width, port_to.data_width, burst_cycles)
         ecc_rdata = BufferizeEndpoints({"source": DIR_SOURCE})(ecc_rdata)
         self.submodules += ecc_rdata
         self.comb += [
@@ -142,3 +170,16 @@ class LiteDRAMNativePortECC(Module, AutoCSR):
                 )
             )
         ]
+        if with_we_error_detection:
+            we_errors = self.we_errors.status
+            self.sync += [
+                If(self.clear.re,
+                    we_errors.eq(0),
+                ).Else(
+                    If(we_errors != (2**len(we_errors) - 1),
+                        If(ecc_wdata.we_error,
+                            we_errors.eq(we_errors + 1)
+                        )
+                    )
+                )
+            ]
