@@ -1,6 +1,9 @@
-# This file is Copyright (c) 2012 Sebastien Bourdeauducq <sb@m-labs.hk>
-# This file is Copyright (c) 2012-2019 Florent Kermarrec <florent@enjoy-digital.fr>
-# License: BSD
+#
+# This file is part of LiteDRAM.
+#
+# Copyright (c) 2012 Sebastien Bourdeauducq <sb@m-labs.hk>
+# Copyright (c) 2012-2019 Florent Kermarrec <florent@enjoy-digital.fr>
+# SPDX-License-Identifier: BSD-2-Clause
 
 # 1:2 and 1:4 frequency-ratio DDR / LPDDR / DDR2 / DDR3 PHYs for Spartan-6
 #
@@ -54,8 +57,6 @@ class S6HalfRateDDRPHY(Module):
                 nphases       = nphases,
                 rdphase       = 0,
                 wrphase       = 1,
-                rdcmdphase    = 1,
-                wrcmdphase    = 0,
                 cl            = 5,
                 cwl           = 6,
                 read_latency  = 6,
@@ -71,8 +72,6 @@ class S6HalfRateDDRPHY(Module):
                 nphases       = nphases,
                 rdphase       = 0,
                 wrphase       = 1,
-                rdcmdphase    = 1,
-                wrcmdphase    = 0,
                 cl            = 3,
                 read_latency  = 5,
                 write_latency = 0
