@@ -93,20 +93,25 @@ class USDDRPHY(Module, AutoCSR):
 
         # PHY settings -----------------------------------------------------------------------------
         self.settings = PhySettings(
-            phytype       = phytype,
-            memtype       = memtype,
-            databits      = databits,
-            dfi_databits  = 2*databits,
-            nranks        = nranks,
-            nphases       = nphases,
-            rdphase       = self._rdphase.storage,
-            wrphase       = self._wrphase.storage,
-            cl            = cl,
-            cwl           = cwl,
-            read_latency  = cl_sys_latency + 5,
-            write_latency = cwl_sys_latency - 1,
-            cmd_latency   = cmd_latency,
-            cmd_delay     = cmd_delay,
+            phytype                   = phytype,
+            memtype                   = memtype,
+            databits                  = databits,
+            dfi_databits              = 2*databits,
+            nranks                    = nranks,
+            nphases                   = nphases,
+            rdphase                   = self._rdphase.storage,
+            wrphase                   = self._wrphase.storage,
+            cl                        = cl,
+            cwl                       = cwl,
+            read_latency              = cl_sys_latency + 5,
+            write_latency             = cwl_sys_latency - 1,
+            cmd_latency               = cmd_latency,
+            cmd_delay                 = cmd_delay,
+            write_leveling            = True,
+            write_latency_calibration = True,
+            read_leveling             = True,
+            delays                    = 512,
+            bitslips                  = 8,
         )
 
         if is_rdimm:
